__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *
# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def gofish_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"                        # 19 Product_Image
    vendor_image = "-1"                 # 20 Vendor_Image

    # Finding Product Name

    divmb = soup.find('div', {'class': "p-3 mb-1 fs-3 fw-bold border border-2 bg-white rounded"})
    if divmb is None:
        divmb = soup.find('div', {'class': "p-3 mb-1 fs-4 fw-bold border border-2 bg-white rounded"})

    name = divmb.text
    name = name.replace('\n', ' ')
    name = name.replace('\r', ' ')
    name = name.replace('\t', ' ')
    name = name.replace(",", "")
    name = name.strip()

    # Finding Vendor
    vendor = soup.find('div', {'class': 'my-1'}).find('a').text.strip()

    # Finding Vendor Rating
    # temp = soup.find('div', {'class': ""}).text
    # temp = temp.split('(')
    # rating = temp[0].replace("Vendor's Review : ", "")
    # rating = rating.replace("%", "")
    # rating_vendor = rating.strip()

    # Finding the Product Rating and Number of Product Reviews
    # reviews = temp[2].replace(" review)", "")
    # reviews = reviews.strip()

    # temp = temp[1].split(")")
    # rating = temp[1].replace("Product Review : ", "")
    # rating = rating.replace("%", "")
    # rating_item = rating.strip()

    # Finding Prices
    precios = soup.findAll('td', {'class': "text-end text-nowrap"})
    USD = precios[0].text.strip().replace('$', '')

    # Finding the Product Category
    # pmb = soup.findAll('p', {'class': "mb-1"})

    # category = pmb[-1].text
    # category = category.replace("Category: ", "").strip()

    # Finding the Product Quantity Available
    # left = divmb[-1].text
    # left = left.split(",", 1)[1]
    # left = left.replace("in stock", "")
    # left = left.strip()

    # Finding Number Sold
    # sold = divmb[-1].text
    # sold = sold.split(",", 1)[0]
    # sold = sold.replace("sold", "")
    # sold = sold.strip()

    # Finding Shipment Information (Origin)
    origin = soup.findAll('div', {'class': "p-3 mt-2 mb-3 border border-2 bg-white rounded"})
    remove = origin[0].find('span').text.strip()
    origin = origin[0].text.strip()
    origin = origin.replace(remove, '')

    shipFrom = origin.strip()

    # Finding Shipment Information (Destination)
    dest = soup.findAll('div', {'class': 'p-3 mb-3 overflow-auto border border-2 bg-white rounded'})
    dest = dest[-1].text.strip()
    dest = dest.replace('[', '')
    dest = dest.replace(']', '')

    shipTo = dest[1:].strip()

    # Finding the Product description
    cardbody = soup.findAll('div', {'class': "p-3 mb-3 overflow-auto border border-2 bg-white rounded"})
    describe = cardbody[0].text
    describe = describe.replace('\n', ' ')
    describe = describe.strip()

    # Finding Product Image
    img = soup.findAll('figure', {'class': 'image-feature'})[0]
    image = img.find('img', {'class': 'image-block rounded'})
    image = image.get('src')
    image = image.split('base64,')[-1]

    # Searching for CVE and MS categories
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def gofish_listing_parser(soup):

    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "GoFish"                     # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft = []                                # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []                                # 20 Product_Image
    image_vendor = []                         # 21 Vendor_Image
    href = []                                 # 22 Product_Links

    listing = soup.find('tbody', {"class": "border border-2 align-middle"}).findAll('tr')

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        href.append(link)

        # Finding the Product
        product = bae[1].text
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.replace("...", "")
        product = product.strip()
        name.append(product)

        # Finding Product Image
        product_image = bae[0].find('img')
        product_image = product_image.get('src')
        product_image = product_image.split('base64,')[-1]
        image.append(product_image)

        # Finding Prices
        price = a.find('span', {"class": "fw-bold text-nowrap"}).text
        price = price.replace("$","")
        price = price.strip()
        USD.append(price)

        # Finding the Vendor
        vendor_name = bae[-1].text
        vendor_name = vendor_name.replace(",", "")
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        image_vendor.append("-1")

        # Finding the Category
        # cat = lb[-1].find("span").text
        # cat = cat.replace("class:", "")
        # cat = cat.strip()
        # category.append(cat)

        # span = lb[0].findAll("span")

        # Finding Number of Views
        # num = span[0].text
        # num = num.replace("views:", "")
        # num = num.strip()
        # sold.append(num)

        # Finding Number Sold
        # num = span[2].text
        # num = num.replace("Sold:", "")
        # num = num.strip()
        # sold.append(num)

        # Finding Quantity Left
        # quant = span[1].text
        # quant = quant.replace("stock:", "")
        # quant = quant.strip()
        # qLeft.append(quant)

        # add shipping information
        # ship = lb[2].findAll('small')[1].findAll('span')[1].text.split("->")
        # shipFrom.append(ship[0].replace("Ship from ", "").strip())
        # shipTo.append(ship[1].replace("to ", "").strip())


        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image, image_vendor)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def gofish_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.find('tbody', {'class': 'border border-2 align-middle'})
    listing = soup.findAll('tr')
    listing = listing[1:]
    # for a in listing:
    #     bae = a.find('a', {"class": "text-info"}, href=True)
    #     link = bae['href']
    #     href.append(link)

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        href.append(link)

    return href